% % MatlabSampleRecon2
%
% Script to illustrate use of Matlab for reconstruction of Feb-Sep mean temperature from PCs 
% of four Poland site chronologies. Stopping step for stepwise regression of final model is
% selected by split-sample validation. Because the sample size for
% calibration is only half of the full calibration size with split-sample
% validation, pin and pout  of 0.05 and 0.10 will not let more than one PC
% enter the model. I had to change those (see below) to 0.20 and 0.40) to
% get the final model to have PC1 and PC4 in it. 
%
% This script has following steps:
%
% -Reads in the monthly T data and call Meko Matlab function seaspt to seasonalize to
%   Feb-Sept average
%- Reads in a time series matrix (tsm) of four residual site chronologies: 
%   1) WEG Abies alba, 2) Robert's oak, 3) Picea (Pola022), and 4) another Abies Alba
%   chronology (Pola024)
%- Calls Meko Matlab function eigen1 for PCA on the chronologies and to produce the
%   four time series PC1, PC2, PC3 and PC4
%- Specify calibration period 1940-2004 
%- Calls Meko Matlab function reconsw3 to do stepwise regression of y on x1, x2,
%   x3, x4 and generate a reconstruction, yhat, covering 1836-2004
%- Makes a few figure windows, including one with a time plot ot the
%   reconstruction and its 50% confidence interval
%
% The results here may not exactly match the results Group II got because I
% the exact input tree-ring chronologies used by Group II might differ from
% the versions I use here for illustration. Also, in the class we first did
% a calibration on the full set, and then used the selected predictors for
% the split-sample validation. This is really pseudo-validation, because
% the selected predictors are biased to favor a signal on both halves of
% the data even before starting the split-sample validation. The procedure
% used in this script, and by function reconsw3, is different, and applies
% split-sample calibration validation without prior selection of the set of
% predictors. 
%
% At last, you will see 11 figure windows.  Windows 1-5 are for analysis of
% residuals. Those windows are generated in function residck1, called by reconsw1.
% Windows 6-8 are made by function rcsplit1. Remaining plots are made by this 
% script. A list of the figure windows:
%
% 1 Time plots of observed (yo) and reconstructed (yh) predicand (top), and
% of the regression residuals (yo-yh) (bottom). A trend line is fit in the
% bottom, with a test for significance of linear trend in residuals.
% 
% 2 Histogram of regression residuals (like this to look normal)
%
% 3 Autocorrelation function of regression residuals
%
% 4 Scatterplot of residuals at time t against residuals at time t-1, with
% ftted line. This is another way of looking a lag-1 persistence in
% residuals
%
% 5  Scatterplot of residuals against fitted values (predicted values of
% predictand). This plot used to judge mainly whether any pattern of
% dependence of variance of residuals on the predictions. For example, a
% fanning out left to right could indicate variance of residuals not
% constant, but a function of sized of fitted values.
%
% 6  Time plots of observed and predicted y for the full calibration
% period, along with annotated R-squared (identical to top plot in Fig 1)
%
% 7  Time plots comparing split-sample calibrations and prediction. Let A
% be the early half and B the late half. Top shows agreement of observed
% with calibration predictions when calibration period is A. Bottom shows
%  agreement of predictions from a model calibrated on A with observations
% for on period B.
%
% 8  Similar time plots as in 7, except now the focus is on model
% calibrated on second half (B)
%
% 9 Time plots of the four original input chronologies before conversion by
% PCA. This plot can be used to check that no mistakes in reading in the
% chronologies
%
% 10 Bar chart of percentage of tree-ring variance in each PC.
%
% 11 Time plot of the reconstruction with its 50% confidence interval. This
% confidence interval used the variance of the split-sample validation
% residuals and assumes they are normally distributed. 

clear all
close all
clc


%--- HARD CODE

%---- READ IN THE MONTHLY T DATA AND SEASONALIZE TO FEB-SEPT

% Use Matlab's "load" function, which works with ascii txt data
flnm = 'tempfrom1929.dat';
D=load(flnm,'-ascii'); % D is 13-col, with year and Jan-dec data in deg C

% set up input to call seaspt
begmo=2; endmo=9;  % Feb and Sept as start and end month
k = 2; % option specifying the seasonalizing is by average rather than total
F = seaspt(D,begmo,endmo,k); % year as col 1, Feb-Sep T as col 2

% Let y1 be the temperature series, and yry1 be the corresponding vector of
% years
yry1 = F(:,1);
y1 = F(:,2);


%---- READ IN THE RESIDUAL SITE CHRONOLOGIES
%
% Here, will use function tblread, which reads tabular data assumed to have
% case names in column 1 (these are the years, and variable names in row 1
% above the other columns

flnm = 'Res_chronologies.txt'; % this is the input file
[X1, Xnms, CaseNames] = tblread(flnm,'tab'); % X1 is 4-col matrix of the chronologies

% convert the CaseNames from string to number-- this will be column vector of
% years
yrX1 = str2num(CaseNames); % col vector of years

% Xnms is a character matrix with site ids in each row; convert those to a
% cell variable, with each member an id
nms = cellstr(Xnms);

% Make a quick plot of the chronologies
xlims=[min(yrX1) max(yrX1)]; % for setting the limits of x axis



%---- PCA ON THE SITE CHRONOLOGIES
%
% Here, will use my function eigen1, which calls various built-in matlab
% functions. We will do the PCA on the correlation matrix rather than the
% covariance matrix of the chronologies

k = 1; % specify use the correlation matrix for the PCA
[R,V,L,S,G,B]=eigen1(X1,k); % returns correlation matrix in R, loadings of PCs in V, eigenvalues in L,
%  and time series of PC scores (PC1, PC2, PC3, PC4) in G
yrG = yrX1;

% From the eigenvalues, compute the percentage of variance in each PC
p = 100*diag(L)/sum(diag(L));




%-- REGRESSION AND RECONSTRUCTION 
%
% Done with call to reconsw1 (reconstruction stepwise 1). Set up the input
% arguments for call to reconsw1.

%  Set up X and y, the time series for the overlap, defined by default as the
%  calibration period, but wanted by us 1940-2004
yrgo =1940;
yrsp =2004;

% L = yrG>=yrgo & yrG<=yrsp;
% X = G(L,:);
% yrX = yrG(L);
% 
% L = yry1>=yrgo & yry1<=yrsp;
% y = y1(L,:);
% yry = yry1(L);

%  


% specs = structure of specifications
specs.pin=0.15; %   .pin for entry in stepwise (see notes for special case)
specs.pout=0.30; %  for removal in stepwise (see notes for special case)
specs.nlag=0; %   .nlag: maximum negative lag on predictors (
specs.plag=0; %   .plag: maximum positive lag allowed on predictors
specs.CalYrs = [yrgo yrsp] ; %   .CalYrs (1x2)i or []:   a specified first and last year (on y) of
%      calibration period; if [], function will use longest possible calib  period
specs.LRA= [1 1]; %  : analysis of residual (1x2)L
%       (1) true==do it;   false=skip it  (1 is true, 0 is false)
%       (2) true==do graphics (slower); false=skip graphics [this option
%       meaningless if LRA(1) true
specs.klong=1;  % if odd number of years in full calibration, early will be assigned as longer of the two

% SpecsSim: specifications for call to pdgmsim from reconsw1
SpecsSim.nsim=1000; % (1x1)i number of simulations (e.g., 1000)
SpecsSim.mtaper=0.10; %  (1x1)r  decimal fractional total of series to be tapered (e.g.,
%        0.10, which means 5% on front and 5% on back
SpecsSim.kopt=[1 2]; % (1x2)i options
%       (1) force sims to have same mean and standard deviation as the
%               observed series
%           ==1 yes;  ==2 no
%       (2)  turn off the msgbox warning about failing Lilliefors
%           ==1 yes;  ==2 no (default if length(kopt)==1)
SpecsSim.len=[]; %  (1x1)i  or []. desired length of simulations; or, if [], same
%           length as the observed series

%  Call to reconsw1 to do stepwise reconstruct
%Result=reconsw1(X,yrX,y,yry,specs,SpecsSim);
Result=reconsw3(G,yrG,y1,yry1,specs,SpecsSim);



%---- COMPUTE 50% CONFIDENCE INTERVAL FOR RECONSTUCTION USING
% SPLIT-SMPLE PSEUDEO-VALIDATION RESIDUALS. ASSUMING RESIDUALS NORMALLY DISTRIBUTED, THIS
% INTERVAL IS   yhat +- 0.67449 RMSEv, where yhat is the reconstructed
% temperature, and RMSEv is the root-mean-square error of cross-validation
% residuals

yh=Result.yh; % reconstructed Feb-Sep temperature
yryh = Result.yryh; % year vector for yh
c = Result.statsv.rmse(1,2) ; % psuedo-validation RMSE;  The RMSE used here is based on residuals 
% generated as  follows, where C, A, B are full calibration period, early
% half, and late half:
% 1 Find stopping step using split-sample calibration validation
%   1.1 Calibrate on A, validate on B, 
%   1.2 Calibrate on B, validate on A
%   1.3 Find the highest step in 1.1 and 1.2 for which 1) RMSEv has not
%       increased from previous step for either of the two halves and 2) RE>0 for
%       for both halves;   call this step m.
% 2 Repeat stepwise on C, specifying m as the highest allowable step
%     (the stopping step); this yields some specic set of predictors.
% 3 Using those specified predictors, repeat the split sample calibration-validation (not stepwise)
% 4 Using the model calibrated on A, predict y for B, and compute the series of validation residuals
%    for B as observed-predicted. 
% 5 Using the model calibrated on B, predict y for A, and compute the series of validation residuals
%    for A as observed-predicted. 
% 6 Splice the residuals for A and B from the previous two steps into a
%   single time series of validation residuals, e
% 7  Compute the sum of squares of e, divide by the series length to get
%   the mean square, and take the square root to get the RMSEv

yhhi = yh + 0.67449*c; % upper confidence interval
yhlo = yh - 0.67449*c; % lower confidence interval


%---- MISCELLANEOUS FIGURES 

%-----  Time plots of original chronologies for their common period
figure(9);
h = plot(yrX1,X1);
set(gca,'XLim',xlims) ; % set x-axis to start and end at data limits of years
legend(nms)
xlabel('Year')
ylabel('Index')
title('Residual site chronologies')


%----- Bar chart of percentage of tree-ring variance explained by PCs
figure(10)
h = bar(p);
xlabel ('PC#')
ylabel('% variance explained')
title('Tree-ring percentage variance explained by each PC')

%---- Time plot or reconstruction with 50% confidence interval
figure (11)
[cL,cB,cW,cH]=figsize(.8,.6);
set(gcf,'Position',[cL cB cW cH]);

stryrs=[num2str(yryh(1)) '-'  num2str(yryh(end))]; % make string for labeling years in title

xlims =[yryh(1) yryh(end)]; % want as x-axis  limits on plot
h = plot(yryh,yh,yryh, yhlo, yryh,yhhi,[yryh(1) yryh(end)],[mean(yh) mean(yh)]); % make the plot
set(h(1),'color',[0 0 1]);
set(h(2),'Color',[1 0 0]);
set(h(3),'Color',[1 0 0]);
set(h(4),'Color',[0 0 0]);
set(gca,'XLim',xlims); % set x-axis limits

xlabel('Year');
ylabel('T (deg C)')
title(['Reconstructed T, ' stryrs ', with 50% confidence interval'])


%---- MESSAGE ABOUT HOW TO ACCESS DATA AND STATISTICS

str_msg1 = {['Structure variable ''Results'' has data and statistics for the regression and reconstruction'],...
    'You can access this data from the Matlab command prompt by typing ''Result''',...
    'For example, Result.yh and Result.yryh are the reconstruction and its year vector.',...
    'Fields Result.statsc and Results.statsv hold the statistics for calibration and validation.',...
    '     For example, type ''Result.statsc'' and you will see the regression R2 and other statistics.',...
    '     Type ''Result.statsv'' and you will see RE and other validation statistics'};
 uiwait(msgbox(str_msg1,'Message','modal'));
