% driver_seascorr  --- run seascorr in "input argument" mode on test data
%
% This script sets up input arguments and calls function seascorr to run on 
% the sample data used in the Computers & Geosciences manuscript.
% You may compare the results to outputs available in the online
% supplementary materials of that journal as a check.  (Be aware that
% significance thresholds can differ from run to run in Monte Carlo
% methods.)

clear all;
close all;
clc;

kcolor=menu('Choose','Color','B/W');

%---- READ TREE-RING DATA
% 
% seascorr requires 2 column vectors, the tree ring data in x, and the year
% in yrx
file_tree='sad_std.txt'; % Tree-ring data in 2-col format
D=dlmread(file_tree);
%[x,s,yrx]=crn2vec2(file_tree); % reads a crn file, returns data in x, year in yrx,
% sample size in s

yrx=D(:,1);
x=D(:,2);
x=0.001*x; % scale tree-ring index (data in txt file does not have decimal pt)


%---- READ CLIMATE DATA
% 
% The data are in 13-col ascii files, but seascorr in "input-argument" mode
% expects 3-column format (year, month, data)
% Must convert the data.  
% Function c132tss converts data from 13-col ascii to the desired form
file_P='7.5-9.5E_34.5-36.5N_ppt.txt'; % primary climate variable, 13-column ascii
file_T='7.5-9.5E_34.5-36.5N_temp.txt'; % secondary climate variable, 13-column ascii

% Primary climate variable
D = dlmread(file_P); % 13-col numeric matrix, year as col 1
if size(D,2)==14; % depending on line-end character, dlmread can append a column of zeros
    D(:,end)=[];
else
end
D = c132tss(D); % D.tsm is a 3-col matrix (year, month, data)
P=D.tsm; % rename

% Seconndary climate variable
D = dlmread(file_T); % 13-col numeric matrix, year as col 1
if size(D,2)==14; % depending on line-end character, dlmread can append a column of zeros
    D(:,end)=[];
else
end

D = c132tss(D); % D.tsm is a 3-col matrix (year, month, data)
T=D.tsm; % rename



%--- SPECIFY SEASONAL GROUPING AND NUMBER OF SIMULATIONS

endmo =9;  % September as the month marking end of growth year
g ={1,3,9,12};  % 1-month, 3-month, 9-monthe and 12-month "seasons"Result = seascorr(x,yrx,P,T,endmo,g,kopt,textinfo,nsim);
nsim=1000;

%--- SET OPTIONS

kopt=[kcolor 1];
% kopt(1) == 1 specifies color rather than B/W plots
% kopt(2)==1 specifies P will be primary climate variable and T secondary


%--- SET TEXT INFORMATION ON CLIMATE SERIES

textinfo{1} ='Precipitation'; % name of first climate series
textinfo{2}='P'; % one-letter code...
textinfo{3}='mm'; % units
textinfo{4}='sum'; % seasonalize by summing over months

textinfo{5} ='Temperature'; % name of first climate series
textinfo{6}='T'; % one-letter code...
textinfo{7}='deg C'; % units
textinfo{8}='mean'; % seasonalize by averaging over months

%time_specs={[],[1930 1950],[1963 1990]};
time_specs={[1903  2002],[1903 1952],[1953 1963]}; % full-period, sub-periods, 
%   minimum allowable length of period for correlation


%---- Some minimal checking to avoid invalid specification of time periods
% (seascorr will also apply a check to the periods)
t1=time_specs{1};
t2=time_specs{2};
t3=time_specs{3};
if ~isempty(t1) && ~isempty(t2);
    if ~(t2(1)>=t1(1) && t3(1)>=t1(1))
        error('Do not specify a sub-period to start before the specified start year of full period');
    end
    if ~(t2(end)<=t1(end) && t3(end)<=t1(end))
        error('Do not specify a sub-period to end after the specified end year of full period');
    end
    if t2(2)>=t3(1);
        error('Do not let the two sub-periods overlap');
    end
end

%--- CALL SEASCORR

t=cputime; % allows checking cpu time required to run function (type "tcpu" at Matlab prompt after running this
% script
Result=seascorr(x,yrx,P,T,endmo,g,kopt,textinfo,nsim,time_specs);
tcpu=cputime-t;